"""
A simple object relational mapper for SQLite.

Example:
    from pylaxz.orm import Database
    db = Database('path/to/sqlite.db')
    
    # create Model
    class TestModel(db.Model):
        text = str
        is_ok = bool
        def __init__(self, text, is_ok):
            self.text = text
            self.is_ok = is_ok
    
    obj = TestModel('Some String',True)
    obj.save()
    db.commit()

"""
import sqlite3

#: Dictionary to map Python and SQLite data types
DATA_TYPES = {
    str: 'TEXT',
    int: 'INTEGER',
    float: 'REAL',
    bool: 'BOOL'}


def attrs(obj):
    """ Return attribute values dictionary for an object """
    return dict(i for i in vars(obj).items() if i[0][0] != '_')


def copy_attrs(obj, remove=None):
    """ Copy attribute values for an object """
    if remove is None:
        remove = []
    return dict(i for i in attrs(obj).items() if i[0] not in remove)


def render_column_definitions(model):
    """ Create SQLite column definitions for an entity model """
    model_attrs = attrs(model).items()
    model_attrs = {k: v for k, v in model_attrs if k != 'db'}
    return ['%s %s' % (k, DATA_TYPES[v]) for k, v in model_attrs.items()]


def render_create_table_stmt(model):
    """ Render a SQLite statement to create a table for an entity model """
    sql = 'CREATE TABLE {table_name} (id integer primary key autoincrement, {column_def});'  # noqa
    column_definitions = ', '.join(render_column_definitions(model))
    params = {'table_name': model.__name__, 'column_def': column_definitions}
    return sql.format(**params)


class Database(object):  # pylint: disable=R0205
    """ Proxy class to access sqlite3.connect method """

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self._connection = None
        self.connected = False
        self.Model = type('Model%s' % str(self), (Model,), {
                          'db': self})  # pylint: disable=C0103

    @property
    def connection(self):
        """ Create SQL connection """
        if self.connected:
            return self._connection
        self._connection = sqlite3.connect(*self.args, **self.kwargs)
        self._connection.row_factory = sqlite3.Row
        self.connected = True
        return self._connection

    def close(self):
        """ Close SQL connection """
        if self.connected:
            self.connection.close()
        self.connected = False

    def commit(self):
        """ Commit SQL changes """
        self.connection.commit()

    def execute(self, sql, *args):
        """ Execute SQL """
        return self.connection.execute(sql, args)

    def executescript(self, script):
        """ Execute SQL script """
        self.connection.cursor().executescript(script)
        self.commit()


class Manager(object):  # pylint: disable=R0205
    """ Data mapper interface (generic repository) for models """

    def __init__(self, db, model, type_check=True):
        self.db = db
        self.model = model
        self.table_name = model.__name__
        self.type_check = type_check
        if not self._hastable():
            self.db.executescript(render_create_table_stmt(self.model))

    def all(self):
        """ Get all model objects from database """
        result = self.db.execute('SELECT * FROM %s' % self.table_name)
        return (self.create(**row) for row in result.fetchall())

    def create(self, **kwargs):
        """ Create a model object """
        obj = object.__new__(self.model)
        obj.__dict__ = kwargs
        return obj

    def delete(self, obj):
        """ Delete a model object from database """
        sql = 'DELETE from %s WHERE id = ?'
        self.db.execute(sql % self.table_name, obj.id)

    def get(self, id):
        """ Get a model object from database by its id """
        sql = 'SELECT * FROM %s WHERE id = ?' % self.table_name
        result = self.db.execute(sql, id)
        row = result.fetchone()
        if not row:
            msg = 'Object%s with id does not exist: %s' % (self.model, id)
            raise ValueError(msg)
        return self.create(**row)

    def has(self, id):
        """ Check if a model object exists in database by its id """
        sql = 'SELECT id FROM %s WHERE id = ?' % self.table_name
        result = self.db.execute(sql, id)
        return True if result.fetchall() else False

    def save(self, obj):
        """ Save a model object """
        if 'id' in obj.__dict__ and self.has(obj.id):
            msg = 'Object%s id already registred: %s' % (self.model, obj.id)
            raise ValueError(msg)
        clone = copy_attrs(obj, remove=['id'])
        self.type_check and self._isvalid(clone)
        column_names = '%s' % ', '.join(clone.keys())
        column_references = '%s' % ', '.join('?' for i in range(len(clone)))
        sql = 'INSERT INTO %s (%s) VALUES (%s)'
        sql = sql % (self.table_name, column_names, column_references)
        result = self.db.execute(sql, *clone.values())
        obj.id = result.lastrowid
        return obj

    def update(self, obj):
        """ Update a model object """
        clone = copy_attrs(obj, remove=['id'])
        self.type_check and self._isvalid(clone)
        where_expressions = '= ?, '.join(clone.keys()) + '= ?'
        sql = 'UPDATE %s SET %s WHERE id = ?' % (self.table_name, where_expressions)  # noqa
        self.db.execute(sql, *(list(clone.values()) + [obj.id]))

    def _hastable(self):
        """ Check if entity model already has a database table """
        sql = 'SELECT name len FROM sqlite_master WHERE type = ? AND name = ?'
        result = self.db.execute(sql, 'table', self.table_name)
        return True if result.fetchall() else False

    def _isvalid(self, attr_values):
        """ Check if an attr values dict are valid as specificated in model """
        attr_types = attrs(self.model)
        value_types = {a: v.__class__ for a, v in attr_values.items()}

        for attr, value_type in value_types.items():
            if value_type is not attr_types[attr]:
                msg = "%s value should be type %s not %s"
                raise TypeError(msg % (attr, attr_types[attr], value_type))


class Model(object):  # pylint: disable=R0205
    """ Abstract entity model with an active record interface """

    db = None

    def delete(self, type_check=True):
        """ Delete this model object """
        return self.__class__.manager(type_check=type_check).delete(self)

    def save(self, type_check=True):
        """ Save this model object """
        return self.__class__.manager(type_check=type_check).save(self)

    def update(self, type_check=True):
        """ Update this model object """
        return self.__class__.manager(type_check=type_check).update(self)

    @property
    def public(self):
        """ Return the public model attributes """
        return attrs(self)

    def __repr__(self):
        return str(self.public)

    @classmethod
    def manager(cls, db=None, type_check=True):
        """ Create a database managet """
        return Manager(db if db else cls.db, cls, type_check)



"""	
Small utility for me or your personal use	
Usage:	
------	
    $ pylaxz -flag argument	
Available flag are:	
    -h, --help         Show this help	
    -u, --update       Update available check	
    -v, --version      Show current version	
    -N, --network      Check network infomation	
        Arguments : (ip), (i)nternet, (s)peed, 	
    -S, --system       Check system information	
        Arguments : (os), (os-info), 	
    -L, --lxz      Ingegrate with lxz TODO	
        --help for HELP.	
        	
More information is available at:	
- https://pypi.org/project/py-laxz/	
- https://github.com/minlaxz/py-laxz/	
"""


# Standard library imports
from subprocess import run  # , PIPE, Popen, STDOUT
from sys import argv, stdout, stderr, exit

# from sqlalchemy import create_engine
# from sqlalchemy.orm import sessionmaker
# from .settings.dbcore import Settings

import click
import os

# pylaxz imports
from .utils import logxs, _network, _system
from .__version__ import version
from .utils.logxs import printf

VERBOSE=False

DIR_PATH = os.path.dirname(os.path.realpath(__file__))
SCRIPT_PATH = os.path.join(DIR_PATH, "shells/")
DK_SHELL = "bash " + SCRIPT_PATH + "dkx.sh "
CHECK_SHELL = "bash " + SCRIPT_PATH + "checkx.sh "
HOW_SHELL = "bash " + SCRIPT_PATH + "howx.sh "
OUTLINE_SHELL = "bash " + SCRIPT_PATH + "outlinex.sh "
X_SHELL = "bash " + SCRIPT_PATH + "xx.sh "
SQL_SHELL = "bash " + SCRIPT_PATH + "sqlx.sh "

@click.group()
def main(direct=True):
    """
    Script: pylaxz\n
    Info: This is under developement / by minlaxz
    """
    pass

@main.command()
@click.argument('arg', type=str, required=False)
def version(**kw):
    printf(f"Version: {version}", _int=1)
    print(version)


@main.command()
@click.option('--verbose', '-v', is_flag=True, help="Verbose Extra Option")
@click.argument('arg',type=str, required=False)
@click.argument('container_id',required=False)
def dk(**kw):
    """
    Description: operating with dockers.
    """
    with open(DK_SHELL[5:-1], 'r') as f:
        lines = f.readlines()
    DKX_OPTS=[i[:-2] for i in lines if i.startswith('--')]
    try:
        if kw['arg'] is None:
            __logxsOut(f'{[i[5:] for i in DKX_OPTS]}')
        else:
            if ('--dk-'+kw['arg']) in DKX_OPTS:
                if (kw['container_id'] is not None):
                    __callShell(cmd=DK_SHELL+ '--dk-{0} {1}'.format(kw['arg'],kw['container_id']))
                else:
                    __callShell(cmd=DK_SHELL+ '--dk-{0}'.format(kw['arg']))
    except (KeyError, Exception) as e:
        click.echo(f"Internal Error. {e}")

@main.command()
@click.option('--verbose', '-v', is_flag=True, help="Verbose Extra Option")
@click.argument('arg',type=str, required=False)
def check(**kw):
    """
    Description: operating with system checks.
    """
    with open(CHECK_SHELL[5:-1], 'r') as f:
        lines = f.readlines()
    CHECKX_OPTS=[i[:-2] for i in lines if i.startswith('--')]
    try:
        if kw['arg'] is None:
            __logxsOut(f"{[i[2:] for i in CHECKX_OPTS]}")
        else:
            if ('--'+kw['arg']) in CHECKX_OPTS:
                __callShell(cmd=CHECK_SHELL+ '--{0}'.format(kw['arg']))
    except (KeyError, Exception) as e:
        click.echo(f"Internal Error. {e}")

@main.command()
@click.option('--verbose', '-v', is_flag=True, help="Verbose Extra Option")
@click.argument('arg',type=str, required=False)
def how(**kw):
    """
    Description: How to do that.
    """
    with open(HOW_SHELL[5:-1], 'r') as f:
        lines = f.readlines()
    HOWX_OPTS=[i[:-2] for i in lines if i.startswith('--')]
    try:
        if kw['arg'] is None:
            __logxsOut(f"{[i[6:] for i in HOWX_OPTS]}")
        else:
            if ('--how-'+kw['arg']) in HOWX_OPTS:
                __callShell(cmd=HOW_SHELL+ '--how-{0}'.format(kw['arg']))
    except (KeyError, Exception) as e:
        click.echo(f"Internal Error. {e}")

@main.command()
@click.option('--verbose', '-v', is_flag=True, help="Verbose Extra Option")
@click.argument('arg',type=str, required=False)
def outline(**kw):
    """
    Description: operating with outline VPN.
    """
    with open(OUTLINE_SHELL[5:-1], 'r') as f:
        lines = f.readlines()
    OUTLINEX_OPTS=[i[:-2] for i in lines if i.startswith('--')]
    try:
        if kw['arg'] is None:
            __logxsOut(f"{[i[10:] for i in OUTLINEX_OPTS]}")
        else:
            if ('--outline-'+kw['arg']) in OUTLINEX_OPTS:
                __callShell(cmd=OUTLINE_SHELL+ '--outline-{0}'.format(kw['arg']))
    except (KeyError, Exception) as e:
        click.echo(f"Internal Error. {e}")

@main.command()
@click.option('--verbose', '-v', is_flag=True, help="Verbose Extra Option")
@click.argument('arg',type=str, required=False)
def x(**kw):
    """
    Description: \ x options for minlaxz / 
    """
    with open(X_SHELL[5:-1], 'r') as f:
        lines = f.readlines()
    X_OPTS=[i[:-2] for i in lines if i.startswith('--')]
    try:
        if kw['arg'] is None:
            __logxsOut(f"{[i[4:] for i in X_OPTS]}")
        else:
            if ('--x-'+kw['arg']) in X_OPTS:
                __callShell(cmd=X_SHELL+ '--x-{0}'.format(kw['arg']))
    except (KeyError, Exception) as e:
        click.echo(f"Internal Error. {e}")

# @main.command()
# @click.option('--verbose', '-v', is_flag=True, help="Verbose Extra Option")
# @click.argument('arg',type=str, required=False)
# def sql(**kw):
#     """
#     Description: \ sql options for minlaxz / 
#     """
#     engine = create_engine('sqlite:///settings/commands.db')
#     # Base.metadata.bind = engine
#     Session = sessionmaker(bind=engine)
#     session = Session()
#     proxy = session.query(Settings).all()
#     print(proxy.one_or_none())


def __callShell(cmd):
    if VERBOSE: print(cmd)
    try:
        run(cmd, shell=True, stderr=stderr, stdout=stdout)
    except (KeyboardInterrupt,Exception) as e:
        __logxsOut(f"{e}\nEXIT.")

def __logxsOut(msg):
    if isinstance(msg, list):
        for i in msg: logxs.printf(i, _int=1)
    else:
        logxs.printf(msg, _int=True)


# def _updater():
#     __logxsOut("updating the script...")
#     cmd = "find . -maxdepth 2 -name '*.py' -print0 | xargs -0 sha1sum | sort -h | sha256sum | awk '{print $1}'"
#     __callShell(cmd)

if __name__ == '__main__':
    main(direct=False)


class Length:
    '''Convert length US to MM'''
    _mode = 1  # 'toMM'

    def __init__(self, *args):
        if Length._mode == 1:
            self.cm = args[0]
            self.inch = None
        else:
            self.inch = args[0]
            self.cm = None
        self.calc()

    def show_MM(self):
        return f"{self.inch} in -> {self.cm} cm"

    def show_US(self):
        return f"{self.cm} cm -> {self.inch} in"

    def calc(self):
        if self.cm == None:
            self.cm = self.inch * 2.54

        else:
            self.inch = self.cm / 2.54

    def result(self):
        self.show_MM() if Length._mode == 1 else self.show_US()

    @classmethod
    def convert(cls, *arg):
        return cls(*arg)


# 2.54 cm = 1 in = 10 muu = 4 mat = 25.4 mm
# 1.27 cm = 0.5 in = 5 muu = 2 mat = 12.7 mm
# 0.254 cm = 0.1 in = 1 muu = 0.4 mat = 2.54 mm



import requests
requests.urllib3.disable_warnings()


class OutlineVPN:
    """
    Outline VPN 

    Options
    -------
    0. -h, --help (help for this options)
    1. -r (remove existing user with key)
    2. get (list existing users and keys)
    3. -a (append new user with key)
    """

    def __init__(self) -> object:
        self.API_URL = input('API_URL? : ')
        self.res_list = requests.get(self.API_URL+'/access-keys', verify=False)
        
        if self.res_list:
            print(f'URL is valid and return {self.res_list.status_code}')
        else:
            print(f'An error occurred and return {self.res_list.status_code}')

    def get_lists(self) -> list:
        return self.res_list.json()['accessKeys']

    def get_metrics(self) -> list:
        res = requests.get(self.API_URL+'/metrics/transfer', verify=False)
        return res.json()['bytesTransferredByUserId']

    def post(self) -> bool:
        user_name = input('Enter new username: ')

    def delete(self) -> bool:
        user_id = input('Enter User ID: ')

    def put(self) -> bool:
        pass



#! /usr/bin/python3
import sys
import json
from pylaxz import printf


def overview_metrics():
    printf(json.loads(sys.argv[1:][0]), _int=1)


def detail_information(*arg):
    metrics_data = json.loads(sys.argv[1:][0])

    # parse single {compact} json string
    access_lists = sys.argv[1:][1][1:-1]

    # manipulate for making string to dictionaries
    access_lists = ['{'+i+'}' for i in access_lists.split('},{')]

    # '[{{},{},{},{}}]' remove '{'....'}'
    access_lists[0] = access_lists[0][1:]
    access_lists[-1] = access_lists[-1][:-1]

    # [{},{},{}]
    access_key_dicts = [json.loads(i) for i in access_lists]

    # printf(access_key_dicts)
    id_or_name = arg[0] if len(arg) > 0 else None
    if id_or_name is not None:
        found_flag = False
        var_id = None
        for i in access_key_dicts:
            for k, v in i.items():
                if k == 'id' or k == 'name':
                    if v == id_or_name:
                        if 'dataLimit' in i:
                            limit = i['dataLimit']['bytes'] / (1024**3)
                            printf("DataLimit Found. : {0:.2f} MB".format(
                                limit), _int=1)
                        else:
                            limit = 0
                        found_flag = True
                        var_id = id_or_name if k == 'id' else i['id']
                        printf(json.dumps(i, indent=4, sort_keys=False), _int=1)
                        try:
                            metric_bytes = metrics_data[var_id]/(1024**3)
                            printf("Usage : {0:.2f} MB".format(
                                metric_bytes), _int=1)
                            if limit != 0:
                                print('Key is DISABLED' if metric_bytes >
                                      limit else 'Warning')
                        except KeyError:
                            printf(f'This ID {var_id} has no usage.', _int=1)
        if not found_flag:
            printf(f'ID : {id_or_name} is Deleted or Not Found !', _int=1)
    else:
        for i in access_key_dicts:
            printf(json.dumps(i, indent=4, sort_keys=False), _int=1)


def oneDetails(userID, userName, userPassword, dataTransferred, ss, dataLimit=None):
    printf(f'ID: {userID}, Name: {userName}, Password: {userPassword}, DataTransferred: {dataTransferred}GB\nAccessURL: {ss}', _int=1)


def details(*arg):

    users = arg[0]
    metrics = arg[1]

    for user in users:
        user['dataTransferred'] = metrics[user['id']] / \
            (1024**3) if user['id'] in metrics.keys() else None

    if len(arg) == 3:
        for user in users:
            printf(
                f'{user["id"]}, {user["name"]}, {user["dataTransferred"]}GB', _int=1)
        try:
            userID = input("Enter User ID : ")
            userID = int(userID)
            for user in users:
                for k, v in user.items():
                    if k == 'id' and v == str(userID):
                        oneDetails(user['id'], user['name'],
                                   user['password'], user['dataTransferred'],
                                   user['accessUrl'])
        except ValueError:
            print('Only ID.')
        except KeyboardInterrupt:
            print('Exit.')
    else:
        for user in users:
            if user['dataTransferred']:
                if 'dataLimit' in user.keys():
                    printf(
                        f'{user} Found DataLimit:{user["dataLimit"]["bytes"]/(1024**3)}GB', _int=1, _err=1)
                else:
                    printf(user, _int=1)
            else:
                printf('No Usage. {}'.format(user), _int=1, _err=1)


if __name__ == "__main__":
    if len(sys.argv) == 2:
        overview_metrics()
    elif len(sys.argv) == 3:
        detail_information()
    elif len(sys.argv) == 4:
        detail_information(sys.argv[3])
    else:
        printf('Error few many args : {}'.format(
            len(sys.argv)), _err=True, _int=1)



import sys, os
import urllib.request, json, string, random
from datetime import datetime
import time

referrer = sys.argv[1:][0]

def randomString(length):
	letters = string.ascii_letters + string.digits
	return ''.join(random.choice(letters) for i in range(length))

def randomDigit(length):
	digit = string.digits
	return ''.join((random.choice(digit) for i in range(length)))

url = f'https://api.cloudflareclient.com/v0a{randomDigit(3)}/reg'
def run():
	try:
		install_id = randomString(22)
		body = {"key": "{}=".format(randomString(43)),
				"install_id": install_id,
				"fcm_token": "{}:APA91b{}".format(install_id, randomString(134)),
				"referrer": referrer,
				"warp_enabled": False,
				"tos": datetime.datetime.now().isoformat()[:-3] + "+02:00",
				"type": "Android",
				"locale": "es_ES"}
		data = json.dumps(body).encode('utf8')
		headers = {'Content-Type': 'application/json; charset=UTF-8',
					'Host': 'api.cloudflareclient.com',
					'Connection': 'Keep-Alive',
					'Accept-Encoding': 'gzip',
					'User-Agent': 'okhttp/3.12.1'
					}
		req         = urllib.request.Request(url, data, headers)
		response    = urllib.request.urlopen(req)
		status_code = response.getcode()	
		return status_code
	except Exception as error:
		print(error)	
g = 0
b = 0
while True:
	result = run()
	if result == 200:
		g = g + 1
		os.system('cls' if os.name == 'nt' else 'clear')
		animation = ["[■□□□□□□□□□] 10%","[■■□□□□□□□□] 20%", "[■■■□□□□□□□] 30%", "[■■■■□□□□□□] 40%", "[■■■■■□□□□□] 50%", "[■■■■■■□□□□] 60%", "[■■■■■■■□□□] 70%", "[■■■■■■■■□□] 80%", "[■■■■■■■■■□] 90%", "[■■■■■■■■■■] 100%"] 
		for i in range(len(animation)):
			time.sleep(0.5)
			sys.stdout.write("\r[+] Preparing... " + animation[i % len(animation)])
			sys.stdout.flush()
		print(f"\n[-] WORK ON ID: {referrer}")    
		print(f"[:)] {g} GB has been successfully added to your account.")
		print(f"[#] Total: {g} Good {b} Bad")
		print("[*] After 18 seconds, a new request will be sent.")
		time.sleep(18)
	else:
		b = b + 1
		os.system('cls' if os.name == 'nt' else 'clear')
		print("[:(] Error when connecting to server.")
		print(f"[#] Total: {g} Good {b} Bad")	



from sqlalchemy import Column, ForeignKey, Integer, String
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship
from sqlalchemy import create_engine

Base = declarative_base()

class Settings(Base):
    __tablename__='settings'
    id = Column(Integer, primary_key=True)
    version = Column(String(250), nullable=False)

engine = create_engine('sqlite:///commands.db')
Base.metadata.create_all(engine)
